@@ -166,15 +166,13 @@ char *do_browser(char *path)
 			say_there_is_no_help();
 #endif
 		} else if (func == do_search_forward) {
-			do_filesearch();
-		} else if (func == do_research) {
-			do_fileresearch(TRUE);
-#ifndef NANO_TINY
+			do_filesearch(FORWARD);
+		} else if (func == do_search_backward) {
+			do_filesearch(BACKWARD);
 		} else if (func == do_findprevious) {
-			do_fileresearch(FALSE);
+			do_fileresearch(BACKWARD);
 		} else if (func == do_findnext) {
-			do_fileresearch(TRUE);
-#endif
+			do_fileresearch(FORWARD);
 		} else if (func == do_left) {
 			if (selected > 0)
 				selected--;
@@ -464,10 +462,11 @@ functionptrtype parse_browser_input(int *kbinput)
 {
 	if (!meta_key) {
 		switch (*kbinput) {
-			case ' ':
-				return do_page_down;
+			case DEL_CODE:
 			case '-':
 				return do_page_up;
+			case ' ':
+				return do_page_down;
 			case '?':
 				return do_help_void;
 			case 'E':
@@ -488,11 +487,9 @@ functionptrtype parse_browser_input(int *kbinput)
 			case '/':
 				return do_search_forward;
 			case 'N':
-#ifndef NANO_TINY
 				return do_findprevious;
-#endif
 			case 'n':
-				return do_research;
+				return do_findnext;
 		}
 	}
 	return func_from_key(kbinput);
@@ -659,42 +656,42 @@ void browser_select_dirname(const char *needle)
 	}
 }
 
-/* Set up the system variables for a filename search.  Return -1 or -2 if
- * the search should be canceled (due to Cancel or a blank search string),
- * return 0 when we have a string, and return a positive value when some
- * function was run. */
-int filesearch_init(void)
+/* Prepare the prompt and ask the user what to search for.  If forwards is
+ * TRUE, search forward in the list; otherwise, search backward.  Return -2
+ * for a blank answer, -1 for Cancel, 0 when we have a string, and a
+ * positive value when some function was run. */
+int filesearch_init(bool forwards)
 {
-	int input;
-	char *buf;
+	char *thedefault;
+	int response;
 
+	/* If something was searched for before, show it between square brackets. */
 	if (*last_search != '\0') {
 		char *disp = display_string(last_search, 0, COLS / 3, FALSE);
 
-		buf = charalloc(strlen(disp) + 7);
+		thedefault = charalloc(strlen(disp) + 7);
 		/* We use (COLS / 3) here because we need to see more on the line. */
-		sprintf(buf, " [%s%s]", disp,
+		sprintf(thedefault, " [%s%s]", disp,
 				(strlenpt(last_search) > COLS / 3) ? "..." : "");
 		free(disp);
 	} else
-		buf = mallocstrcpy(NULL, "");
+		thedefault = mallocstrcpy(NULL, "");
 
-	/* This is now one simple call.  It just does a lot. */
-	input = do_prompt(FALSE, FALSE, MWHEREISFILE, NULL, &search_history,
-				browser_refresh, "%s%s", _("Search"), buf);
-
-	/* Release buf now that we don't need it anymore. */
-	free(buf);
+	/* Now ask what to search for. */
+	response = do_prompt(FALSE, FALSE, MWHEREISFILE, NULL, &search_history,
+						browser_refresh, "%s%s%s", _("Search"),
+						!forwards ? _(" [Backwards]") : "", thedefault);
+	free(thedefault);
 
 	/* If only Enter was pressed but we have a previous string, it's okay. */
-	if (input == -2 && *last_search != '\0')
+	if (response == -2 && *last_search != '\0')
 		return 0;
 
-	/* Otherwise negative inputs are a bailout. */
-	if (input < 0)
+	/* Otherwise negative responses are a bailout. */
+	if (response < 0)
 		statusbar(_("Cancelled"));
 
-	return input;
+	return response;
 }
 
 /* Look for the given needle in the list of files.  If forwards is TRUE,
@@ -756,11 +753,12 @@ void findfile(const char *needle, bool forwards)
 	selected = looking_at;
 }
 
-/* Search for a filename. */
-void do_filesearch(void)
+/* Search for a filename.  If forwards is TRUE, search forward in the list;
+ * otherwise, search backward.*/
+void do_filesearch(bool forwards)
 {
 	/* If the user cancelled or jumped to first or last file, don't search. */
-	if (filesearch_init() != 0)
+	if (filesearch_init(forwards) != 0)
 		return;
 
 	/* If answer is now "", copy last_search into answer. */
@@ -775,7 +773,7 @@ void do_filesearch(void)
 		update_history(&search_history, answer);
 #endif
 
-	findfile(answer, TRUE);
+	findfile(answer, forwards);
 }
 
 /* Search again without prompting for the last given search string,
