@@ -68,18 +68,23 @@ bool also_the_last = FALSE;
 int didfind = 0;
 		/* Whether the last search found something. */
 
-int controlleft, controlright, controlup, controldown, controlhome, controlend;
-int controldelete, controlshiftdelete;
+int controlleft, controlright, controlup, controldown;
+int controlhome, controlend;
 #ifndef NANO_TINY
+int controldelete, controlshiftdelete;
+int shiftleft, shiftright, shiftup, shiftdown;
 int shiftcontrolleft, shiftcontrolright, shiftcontrolup, shiftcontroldown;
 int shiftcontrolhome, shiftcontrolend;
 int altleft, altright, altup, altdown;
+int altdelete;
 int shiftaltleft, shiftaltright, shiftaltup, shiftaltdown;
 #endif
 
 #ifdef ENABLED_WRAPORJUSTIFY
-ssize_t fill = -COLUMNS_FROM_EOL;
-		/* The column where we will wrap lines. */
+ssize_t wrap_at = -COLUMNS_FROM_EOL;
+		/* The relative column where we will wrap lines. */
+ssize_t fill = 0;
+		/* The actual column where we will wrap lines, based on wrap_at. */
 #endif
 
 char *last_search = NULL;
@@ -120,8 +125,7 @@ openfilestruct *firstfile = NULL;
 
 #ifndef NANO_TINY
 char *matchbrackets = NULL;
-		/* The opening and closing brackets that can be found by bracket
-		 * searches. */
+		/* The opening and closing brackets that bracket searches can find. */
 char *whitespace = NULL;
 		/* The characters used when visibly showing tabs and spaces. */
 int whitespace_len[2];
@@ -257,6 +261,13 @@ char *rcfile_with_errors = NULL;
 		/* The first nanorc file, if any, that produced warnings. */
 #endif
 
+bool spotlighted = FALSE;
+	/* Whether any text is spotlighted. */
+size_t light_from_col = 0;
+	/* Where the spotlighted text starts. */
+size_t light_to_col = 0;
+	/* Where the spotlighted text ends. */
+
 /* Return the number of entries in the shortcut list for a given menu. */
 size_t length_of_list(int menu)
 {
@@ -275,6 +286,11 @@ size_t length_of_list(int menu)
 #define NOVIEW  FALSE
 #define BLANKAFTER  TRUE    /* A blank line after this one. */
 #define TOGETHER  FALSE
+#ifdef ENABLE_MULTIBUFFER
+#define CAN_OPEN_OTHER_BUFFER TRUE
+#else
+#define CAN_OPEN_OTHER_BUFFER FALSE
+#endif
 
 /* Empty functions, for the most part corresponding to toggles. */
 void case_sens_void(void)
@@ -445,9 +461,7 @@ functionptrtype func_from_key(int *kbinput)
 void assign_keyinfo(sc *s, const char *keystring, const int keycode)
 {
 	s->keystr = keystring;
-	s->meta = (keystring[0] == 'M' && keystring[2] != '\xE2');
-
-	assert(strlen(keystring) > 1 && (!s->meta || strlen(keystring) > 2));
+	s->meta = (keystring[0] == 'M' && keycode == 0);
 
 	if (keycode)
 		s->keycode = keycode;
@@ -560,6 +574,7 @@ void shortcut_init(void)
 	const char *mark_gist = N_("Mark text starting from the cursor position");
 	const char *copy_gist =
 		N_("Copy current line (or marked region) and store it in cutbuffer");
+	const char *zap_gist = N_("Throw away the current line (or marked region)");
 	const char *indent_gist = N_("Indent the current line (or marked lines)");
 	const char *unindent_gist = N_("Unindent the current line (or marked lines)");
 	const char *undo_gist = N_("Undo the last operation");
@@ -586,7 +601,7 @@ void shortcut_init(void)
 #ifndef NANO_TINY
 	const char *bracket_gist = N_("Go to the matching bracket");
 #endif
-#ifdef ENABLE_HELP
+#if !defined(NANO_TINY) || defined(ENABLE_HELP)
 	const char *scrollup_gist =
 		N_("Scroll up one line without moving the cursor textually");
 	const char *scrolldown_gist =
@@ -643,9 +658,9 @@ void shortcut_init(void)
 	const char *regexp_gist =
 		N_("Toggle the use of regular expressions");
 #ifdef ENABLE_HISTORIES
-	const char *prevhistory_gist =
+	const char *older_gist =
 		N_("Recall the previous search/replace string");
-	const char *nexthistory_gist =
+	const char *newer_gist =
 		N_("Recall the next search/replace string");
 #endif
 #ifndef NANO_TINY
@@ -719,11 +734,7 @@ void shortcut_init(void)
 				N_("Read File"), WITHORSANS(readfile_gist), BLANKAFTER,
 				/* We allow inserting files in view mode if multibuffer mode
 				 * is switched on, so that we can view multiple files. */
-#ifdef ENABLE_MULTIBUFFER
-				VIEW);
-#else
-				NOVIEW);
-#endif
+				CAN_OPEN_OTHER_BUFFER);
 	} else {
 #ifdef ENABLE_JUSTIFY
 		add_to_funcs(do_justify_void, MMAIN,
@@ -765,14 +776,9 @@ void shortcut_init(void)
 		add_to_funcs(do_justify_void, MMAIN,
 				N_("Justify"), WITHORSANS(justify_gist), TOGETHER, NOVIEW);
 #endif
-
 #ifdef ENABLE_SPELLER
 		add_to_funcs(do_spell, MMAIN,
-				N_("To Spell"), WITHORSANS(spell_gist), TOGETHER, NOVIEW);
-#endif
-#ifdef ENABLE_COLOR
-		add_to_funcs(do_linter, MMAIN,
-				N_("To Linter"), WITHORSANS(lint_gist), TOGETHER, NOVIEW);
+				N_("To Spell"), WITHORSANS(spell_gist), BLANKAFTER, NOVIEW);
 #endif
 	}
 
@@ -812,6 +818,12 @@ void shortcut_init(void)
 	add_to_funcs(flip_replace, MREPLACE,
 		N_("No Replace"), WITHORSANS(whereis_gist), BLANKAFTER, VIEW);
 
+#ifdef ENABLE_HISTORIES
+	add_to_funcs(get_history_older_void, MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE,
+		N_("Older"), WITHORSANS(older_gist), TOGETHER, VIEW);
+	add_to_funcs(get_history_newer_void, MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE,
+		N_("Newer"), WITHORSANS(newer_gist), BLANKAFTER, VIEW);
+#endif
 #ifdef ENABLE_JUSTIFY
 	add_to_funcs(do_full_justify, MWHEREIS,
 		N_("FullJstify"), WITHORSANS(fulljustify_gist), TOGETHER, NOVIEW);
@@ -839,6 +851,7 @@ void shortcut_init(void)
 		/* TRANSLATORS: This starts a backward search. */
 		N_("Where Was"), WITHORSANS(wherewas_gist), TOGETHER, VIEW);
 	add_to_funcs(do_findprevious, MMAIN|MHELP|MBROWSER,
+		/* TRANSLATORS: This refers to searching the preceding occurrence. */
 		N_("Previous"), WITHORSANS(findprev_gist), TOGETHER, VIEW);
 	add_to_funcs(do_findnext, MMAIN|MHELP|MBROWSER,
 		N_("Next"), WITHORSANS(findnext_gist), BLANKAFTER, VIEW);
@@ -869,7 +882,7 @@ void shortcut_init(void)
 		N_("Prev Line"), WITHORSANS(prevline_gist), TOGETHER, VIEW);
 	add_to_funcs(do_down, MMAIN|MHELP|MBROWSER,
 		N_("Next Line"), WITHORSANS(nextline_gist), TOGETHER, VIEW);
-#ifdef ENABLE_HELP
+#if !defined(NANO_TINY) || defined(ENABLE_HELP)
 	add_to_funcs(do_scroll_up, MMAIN,
 		N_("Scroll Up"), WITHORSANS(scrollup_gist), TOGETHER, VIEW);
 	add_to_funcs(do_scroll_down, MMAIN,
@@ -915,10 +928,10 @@ void shortcut_init(void)
 	add_to_funcs(do_enter, MMAIN,
 		N_("Enter"), WITHORSANS(enter_gist), BLANKAFTER, NOVIEW);
 
-	add_to_funcs(do_delete, MMAIN,
-		N_("Delete"), WITHORSANS(delete_gist), TOGETHER, NOVIEW);
 	add_to_funcs(do_backspace, MMAIN,
-		N_("Backspace"), WITHORSANS(backspace_gist),
+		N_("Backspace"), WITHORSANS(backspace_gist), TOGETHER, NOVIEW);
+	add_to_funcs(do_delete, MMAIN,
+		N_("Delete"), WITHORSANS(delete_gist),
 #ifndef NANO_TINY
 		TOGETHER,
 #else
@@ -975,15 +988,17 @@ void shortcut_init(void)
 	add_to_funcs(run_macro, MMAIN,
 		N_("Run Macro"), WITHORSANS(runmacro_gist), BLANKAFTER, VIEW);
 
+	add_to_funcs(zap_text, MMAIN,
+		N_("Zap Text"), WITHORSANS(zap_gist), BLANKAFTER, NOVIEW);
+
+#ifdef ENABLE_COLOR
+	if (!ISSET(RESTRICTED))
+		add_to_funcs(do_linter, MMAIN,
+				N_("To Linter"), WITHORSANS(lint_gist), BLANKAFTER, NOVIEW);
+#endif
 	add_to_funcs(do_savefile, MMAIN,
 		N_("Save"), WITHORSANS(savefile_gist), BLANKAFTER, NOVIEW);
 #endif
-#ifdef ENABLE_HISTORIES
-	add_to_funcs(get_history_older_void, MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE,
-		N_("PrevHstory"), WITHORSANS(prevhistory_gist), TOGETHER, VIEW);
-	add_to_funcs(get_history_newer_void, MWHEREIS|MREPLACE|MREPLACEWITH|MWHEREISFILE,
-		N_("NextHstory"), WITHORSANS(nexthistory_gist), BLANKAFTER, VIEW);
-#endif
 
 #ifndef ENABLE_JUSTIFY
 	add_to_funcs(flip_goto, MWHEREIS,
@@ -1044,7 +1059,7 @@ void shortcut_init(void)
 	add_to_funcs(do_page_up, MBROWSER,
 		N_("Prev Page"), WITHORSANS(prevpage_gist), TOGETHER, VIEW);
 	add_to_funcs(do_page_down, MBROWSER,
-		N_("Prev Page"), WITHORSANS(nextpage_gist), TOGETHER, VIEW);
+		N_("Next Page"), WITHORSANS(nextpage_gist), TOGETHER, VIEW);
 
 	add_to_funcs(to_first_file, MBROWSER|MWHEREISFILE,
 		N_("First File"), WITHORSANS(firstfile_gist), TOGETHER, VIEW);
@@ -1074,64 +1089,50 @@ void shortcut_init(void)
 #endif
 
 	/* Link key combos to functions in certain menus. */
-	add_to_sclist(MMOST, "^M", 0, do_enter, 0);
-	add_to_sclist(MMOST, "Enter", KEY_ENTER, do_enter, 0);
+	add_to_sclist(MMOST|MBROWSER, "^M", 0, do_enter, 0);
+	add_to_sclist(MMOST|MBROWSER, "Enter", KEY_ENTER, do_enter, 0);
 	add_to_sclist(MMOST, "^H", 0, do_backspace, 0);
 	add_to_sclist(MMOST, "Bsp", KEY_BACKSPACE, do_backspace, 0);
+	add_to_sclist(MMOST, "Sh-Del", SHIFT_DELETE, do_backspace, 0);
 	add_to_sclist(MMOST, "^D", 0, do_delete, 0);
 	add_to_sclist(MMOST, "Del", 0, do_delete, 0);
 	/* Make ASCII DEL do a delete when requested, otherwise a backspace. */
 	if (ISSET(REBIND_DELETE))
 		add_to_sclist(MMOST, "Del", DEL_CODE, do_delete, 0);
 	else
-		add_to_sclist(MMOST, "Bsp", DEL_CODE, do_backspace, 0);
+		add_to_sclist(MMOST & ~MBROWSER, "Bsp", DEL_CODE, do_backspace, 0);
 	add_to_sclist(MMOST, "^I", 0, do_tab, 0);
 	add_to_sclist(MMOST, "Tab", TAB_CODE, do_tab, 0);
-	add_to_sclist(MMOST & ~MFINDINHELP, "^G", 0, do_help_void, 0);
-	add_to_sclist(MMOST & ~MFINDINHELP, "F1", 0, do_help_void, 0);
+	add_to_sclist((MMOST|MBROWSER) & ~MFINDINHELP, "^G", 0, do_help_void, 0);
 	add_to_sclist(MMAIN|MHELP|MBROWSER, "^X", 0, do_exit, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER, "F2", 0, do_exit, 0);
 	add_to_sclist(MMAIN, "^S", 0, do_savefile, 0);
 	add_to_sclist(MMAIN, "^O", 0, do_writeout_void, 0);
-	add_to_sclist(MMAIN, "F3", 0, do_writeout_void, 0);
 	add_to_sclist(MMAIN, "^R", 0, do_insertfile_void, 0);
-	add_to_sclist(MMAIN, "F5", 0, do_insertfile_void, 0);
 	add_to_sclist(MMAIN, "Ins", 0, do_insertfile_void, 0);
 	add_to_sclist(MMAIN|MHELP|MBROWSER, "^Q", 0, do_search_backward, 0);
 	add_to_sclist(MMAIN|MHELP|MBROWSER, "^W", 0, do_search_forward, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER, "F6", 0, do_search_forward, 0);
 	add_to_sclist(MMAIN, "^\\", 0, do_replace, 0);
 	add_to_sclist(MMAIN, "M-R", 0, do_replace, 0);
-	add_to_sclist(MMAIN, "F14", 0, do_replace, 0);
 	add_to_sclist(MMOST, "^K", 0, do_cut_text_void, 0);
-	add_to_sclist(MMOST, "F9", 0, do_cut_text_void, 0);
 	add_to_sclist(MMOST, "^U", 0, do_uncut_text, 0);
-	add_to_sclist(MMOST, "F10", 0, do_uncut_text, 0);
 #ifdef ENABLE_JUSTIFY
 	add_to_sclist(MMAIN, "^J", 0, do_justify_void, 0);
-	add_to_sclist(MMAIN, "F4", 0, do_justify_void, 0);
 #endif
 #ifdef ENABLE_SPELLER
 	add_to_sclist(MMAIN, "^T", 0, do_spell, 0);
-	add_to_sclist(MMAIN, "F12", 0, do_spell, 0);
-#else
-#ifdef ENABLE_COLOR
-	add_to_sclist(MMAIN, "^T", 0, do_linter, 0);
-	add_to_sclist(MMAIN, "F12", 0, do_linter, 0);
 #endif
+#ifdef ENABLE_COLOR
+	add_to_sclist(MMAIN, "M-B", 0, do_linter, 0);
 #endif
 	add_to_sclist(MMAIN, "^C", 0, do_cursorpos_void, 0);
-	add_to_sclist(MMAIN, "F11", 0, do_cursorpos_void, 0);
 	add_to_sclist(MMAIN, "^_", 0, do_gotolinecolumn_void, 0);
 	add_to_sclist(MMAIN, "M-G", 0, do_gotolinecolumn_void, 0);
-	add_to_sclist(MMAIN, "F13", 0, do_gotolinecolumn_void, 0);
 	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "^Y", 0, do_page_up, 0);
 	add_to_sclist(MHELP|MBROWSER, "Bsp", KEY_BACKSPACE, do_page_up, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "F7", 0, do_page_up, 0);
+	add_to_sclist(MHELP|MBROWSER, "Sh-Del", SHIFT_DELETE, do_page_up, 0);
 	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "PgUp", KEY_PPAGE, do_page_up, 0);
 	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "^V", 0, do_page_down, 0);
 	add_to_sclist(MHELP|MBROWSER, "Space", 0x20, do_page_down, 0);
-	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "F8", 0, do_page_down, 0);
 	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "PgDn", KEY_NPAGE, do_page_down, 0);
 	add_to_sclist(MMAIN|MHELP, "M-\\", 0, to_first_line, 0);
 	add_to_sclist(MMAIN|MHELP, "^Home", CONTROL_HOME, to_first_line, 0);
@@ -1144,7 +1145,6 @@ void shortcut_init(void)
 	add_to_sclist(MMAIN, "M-A", 0, do_mark, 0);
 	add_to_sclist(MMAIN, "^6", 0, do_mark, 0);
 	add_to_sclist(MMAIN, "^^", 0, do_mark, 0);
-	add_to_sclist(MMAIN, "F15", 0, do_mark, 0);
 	add_to_sclist(MMAIN, "M-6", 0, do_copy_text, 0);
 	add_to_sclist(MMAIN, "M-^", 0, do_copy_text, 0);
 	add_to_sclist(MMAIN, "M-}", 0, do_indent, 0);
@@ -1155,8 +1155,9 @@ void shortcut_init(void)
 	add_to_sclist(MMAIN, "M-;", 0, run_macro, 0);
 	add_to_sclist(MMAIN, "M-U", 0, do_undo, 0);
 	add_to_sclist(MMAIN, "M-E", 0, do_redo, 0);
-	add_to_sclist(MMAIN, "M-|", 0, do_cut_prev_word, 0);
+	add_to_sclist(MMAIN, "Sh-^Del", CONTROL_SHIFT_DELETE, do_cut_prev_word, 0);
 	add_to_sclist(MMAIN, "^Del", CONTROL_DELETE, do_cut_next_word, 0);
+	add_to_sclist(MMAIN, "M-Del", ALT_DELETE, zap_text, 0);
 #endif
 #ifdef ENABLE_WORDCOMPLETION
 	add_to_sclist(MMAIN, "^]", 0, complete_a_word, 0);
@@ -1164,51 +1165,51 @@ void shortcut_init(void)
 #ifdef ENABLE_COMMENT
 	add_to_sclist(MMAIN, "M-3", 0, do_comment, 0);
 #endif
-	add_to_sclist(MMOST, "^B", 0, do_left, 0);
-	add_to_sclist(MMOST, "^F", 0, do_right, 0);
+	add_to_sclist(MMOST|MBROWSER, "^B", 0, do_left, 0);
+	add_to_sclist(MMOST|MBROWSER, "^F", 0, do_right, 0);
 #ifdef ENABLE_UTF8
 	if (using_utf8()) {
-		add_to_sclist(MMOST, "\xE2\x97\x80", KEY_LEFT, do_left, 0);
-		add_to_sclist(MMOST, "\xE2\x96\xb6", KEY_RIGHT, do_right, 0);
+		add_to_sclist(MMOST|MHELP|MBROWSER, "\xE2\x97\x80", KEY_LEFT, do_left, 0);
+		add_to_sclist(MMOST|MHELP|MBROWSER, "\xE2\x96\xb6", KEY_RIGHT, do_right, 0);
 		add_to_sclist(MSOME, "^\xE2\x97\x80", CONTROL_LEFT, do_prev_word_void, 0);
 		add_to_sclist(MSOME, "^\xE2\x96\xb6", CONTROL_RIGHT, do_next_word_void, 0);
+#ifndef NANO_TINY
 #ifdef ENABLE_MULTIBUFFER
 		add_to_sclist(MMAIN, "M-\xE2\x97\x80", ALT_LEFT, switch_to_prev_buffer, 0);
 		add_to_sclist(MMAIN, "M-\xE2\x96\xb6", ALT_RIGHT, switch_to_next_buffer, 0);
 #endif
-#ifndef NANO_TINY
 		add_to_sclist(MMAIN|MHELP|MBROWSER, "M-\xE2\x96\xb2", ALT_UP, do_findprevious, 0);
 		add_to_sclist(MMAIN|MHELP|MBROWSER, "M-\xE2\x96\xbc", ALT_DOWN, do_findnext, 0);
 #endif
 	} else
 #endif
 	{
-		add_to_sclist(MMOST, "Left", KEY_LEFT, do_left, 0);
-		add_to_sclist(MMOST, "Right", KEY_RIGHT, do_right, 0);
+		add_to_sclist(MMOST|MHELP|MBROWSER, "Left", KEY_LEFT, do_left, 0);
+		add_to_sclist(MMOST|MHELP|MBROWSER, "Right", KEY_RIGHT, do_right, 0);
 		add_to_sclist(MSOME, "^Left", CONTROL_LEFT, do_prev_word_void, 0);
 		add_to_sclist(MSOME, "^Right", CONTROL_RIGHT, do_next_word_void, 0);
 	}
-	add_to_sclist(MMOST, "M-Space", 0, do_prev_word_void, 0);
-	add_to_sclist(MMOST, "^Space", 0, do_next_word_void, 0);
-	add_to_sclist((MMOST & ~MBROWSER), "^A", 0, do_home, 0);
-	add_to_sclist((MMOST & ~MBROWSER), "Home", KEY_HOME, do_home, 0);
-	add_to_sclist((MMOST & ~MBROWSER), "^E", 0, do_end, 0);
-	add_to_sclist((MMOST & ~MBROWSER), "End", KEY_END, do_end, 0);
+	add_to_sclist(MMOST|MBROWSER, "M-Space", 0, do_prev_word_void, 0);
+	add_to_sclist(MMOST|MBROWSER, "^Space", 0, do_next_word_void, 0);
+	add_to_sclist(MMOST, "^A", 0, do_home, 0);
+	add_to_sclist(MMOST, "Home", KEY_HOME, do_home, 0);
+	add_to_sclist(MMOST, "^E", 0, do_end, 0);
+	add_to_sclist(MMOST, "End", KEY_END, do_end, 0);
 	add_to_sclist(MMAIN|MHELP|MBROWSER, "^P", 0, do_up, 0);
 	add_to_sclist(MMAIN|MHELP|MBROWSER, "^N", 0, do_down, 0);
 #ifdef ENABLE_UTF8
 	if (using_utf8()) {
 		add_to_sclist(MMAIN|MHELP|MBROWSER, "\xE2\x96\xb2", KEY_UP, do_up, 0);
 		add_to_sclist(MMAIN|MHELP|MBROWSER, "\xE2\x96\xbc", KEY_DOWN, do_down, 0);
-		add_to_sclist(MMAIN|MBROWSER, "^\xE2\x96\xb2", CONTROL_UP, do_prev_block, 0);
-		add_to_sclist(MMAIN|MBROWSER, "^\xE2\x96\xbc", CONTROL_DOWN, do_next_block, 0);
+		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^\xE2\x96\xb2", CONTROL_UP, do_prev_block, 0);
+		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^\xE2\x96\xbc", CONTROL_DOWN, do_next_block, 0);
 	} else
 #endif
 	{
 		add_to_sclist(MMAIN|MHELP|MBROWSER, "Up", KEY_UP, do_up, 0);
 		add_to_sclist(MMAIN|MHELP|MBROWSER, "Down", KEY_DOWN, do_down, 0);
-		add_to_sclist(MMAIN|MBROWSER, "^Up", CONTROL_UP, do_prev_block, 0);
-		add_to_sclist(MMAIN|MBROWSER, "^Down", CONTROL_DOWN, do_next_block, 0);
+		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^Up", CONTROL_UP, do_prev_block, 0);
+		add_to_sclist(MMAIN|MBROWSER|MLINTER, "^Down", CONTROL_DOWN, do_next_block, 0);
 	}
 	add_to_sclist(MMAIN, "M-7", 0, do_prev_block, 0);
 	add_to_sclist(MMAIN, "M-8", 0, do_next_block, 0);
@@ -1218,7 +1219,7 @@ void shortcut_init(void)
 	add_to_sclist(MMAIN, "M-)", 0, do_para_end_void, 0);
 	add_to_sclist(MMAIN, "M-0", 0, do_para_end_void, 0);
 #endif
-#ifdef ENABLE_HELP
+#if !defined(NANO_TINY) || defined(ENABLE_HELP)
 	add_to_sclist(MMAIN, "M--", 0, do_scroll_up, 0);
 	add_to_sclist(MMAIN, "M-_", 0, do_scroll_up, 0);
 	add_to_sclist(MMAIN, "M-+", 0, do_scroll_down, 0);
@@ -1271,7 +1272,7 @@ void shortcut_init(void)
 	add_to_sclist(MMAIN, "M-Z", 0, do_toggle_void, SUSPEND);
 #endif /* !NANO_TINY */
 
-	add_to_sclist(((MMOST & ~MMAIN & ~MBROWSER) | MYESNO), "^C", 0, do_cancel, 0);
+	add_to_sclist(((MMOST & ~MMAIN) | MYESNO), "^C", 0, do_cancel, 0);
 
 	add_to_sclist(MWHEREIS|MREPLACE, "M-C", 0, case_sens_void, 0);
 	add_to_sclist(MWHEREIS|MREPLACE, "M-R", 0, regexp_void, 0);
@@ -1309,7 +1310,6 @@ void shortcut_init(void)
 	add_to_sclist(MBROWSER, "^End", CONTROL_END, to_last_file, 0);
 	add_to_sclist(MBROWSER, "^_", 0, goto_dir_void, 0);
 	add_to_sclist(MBROWSER, "M-G", 0, goto_dir_void, 0);
-	add_to_sclist(MBROWSER, "F13", 0, goto_dir_void, 0);
 #endif
 	if (ISSET(TEMP_FILE))
 		add_to_sclist(MWRITEFILE, "^Q", 0, discard_buffer, 0);
@@ -1352,32 +1352,31 @@ void shortcut_init(void)
 	add_to_sclist(MHELP, "Home", KEY_HOME, to_first_line, 0);
 	add_to_sclist(MHELP, "End", KEY_END, to_last_line, 0);
 #endif
+#ifdef ENABLE_COLOR
+	add_to_sclist(MLINTER, "^X", 0, do_cancel, 0);
+#endif
+	add_to_sclist(MMOST & ~MFINDINHELP, "F1", 0, do_help_void, 0);
+	add_to_sclist(MMAIN|MHELP|MBROWSER, "F2", 0, do_exit, 0);
+	add_to_sclist(MMAIN, "F3", 0, do_writeout_void, 0);
+#ifdef ENABLE_JUSTIFY
+	add_to_sclist(MMAIN, "F4", 0, do_justify_void, 0);
+#endif
+	add_to_sclist(MMAIN, "F5", 0, do_insertfile_void, 0);
+	add_to_sclist(MMAIN|MHELP|MBROWSER, "F6", 0, do_search_forward, 0);
+	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "F7", 0, do_page_up, 0);
+	add_to_sclist(MMAIN|MHELP|MBROWSER|MLINTER, "F8", 0, do_page_down, 0);
+	add_to_sclist(MMOST, "F9", 0, do_cut_text_void, 0);
+	add_to_sclist(MMOST, "F10", 0, do_uncut_text, 0);
+	add_to_sclist(MMAIN, "F11", 0, do_cursorpos_void, 0);
+#ifdef ENABLE_SPELLER
+	add_to_sclist(MMAIN, "F12", 0, do_spell, 0);
+#endif
 
 #ifdef DEBUG
 	print_sclist();
 #endif
 }
 
-#if defined(ENABLE_COLOR) && defined(ENABLE_SPELLER)
-/* Assign one function's shortcuts to another function. */
-void replace_scs_for(void (*oldfunc)(void), void (*newfunc)(void))
-{
-	for (sc *s = sclist; s != NULL; s = s->next)
-		if (s->func == oldfunc)
-			s->func = newfunc;
-}
-
-void set_linter_shortcut(void)
-{
-	replace_scs_for(do_spell, do_linter);
-}
-
-void set_speller_shortcut(void)
-{
-	replace_scs_for(do_linter, do_spell);
-}
-#endif /* ENABLE_COLOR && ENABLE_SPELLER */
-
 const subnfunc *sctofunc(const sc *s)
 {
 	subnfunc *f = allfuncs;
@@ -1478,6 +1477,8 @@ sc *strtosc(const char *input)
 	else if (!strcasecmp(input, "copy") ||
 			 !strcasecmp(input, "copytext"))  /* Deprecated.  Remove end of 2018. */
 		s->func = do_copy_text;
+	else if (!strcasecmp(input, "zap"))
+		s->func = zap_text;
 	else if (!strcasecmp(input, "mark"))
 		s->func = do_mark;
 #endif
@@ -1546,7 +1547,7 @@ sc *strtosc(const char *input)
 	else if (!strcasecmp(input, "down") ||
 			 !strcasecmp(input, "nextline"))
 		s->func = do_down;
-#ifdef ENABLE_HELP
+#if !defined(NANO_TINY) || defined(ENABLE_HELP)
 	else if (!strcasecmp(input, "scrollup"))
 		s->func = do_scroll_up;
 	else if (!strcasecmp(input, "scrolldown"))
@@ -1606,9 +1607,9 @@ sc *strtosc(const char *input)
 			 !strcasecmp(input, "gototext"))  /* Deprecated.  Remove end of 2018. */
 		s->func = flip_goto;
 #ifdef ENABLE_HISTORIES
-	else if (!strcasecmp(input, "prevhistory"))
+	else if (!strcasecmp(input, "older"))
 		s->func = get_history_older_void;
-	else if (!strcasecmp(input, "nexthistory"))
+	else if (!strcasecmp(input, "newer"))
 		s->func = get_history_newer_void;
 #endif
 #ifndef NANO_TINY
