@@ -153,7 +153,7 @@ typedef enum {
 } file_format;
 
 typedef enum {
-	HUSH, MILD, ALERT
+	HUSH, NOTICE, MILD, ALERT
 } message_type;
 
 typedef enum {
@@ -178,7 +178,7 @@ typedef enum {
 #ifdef ENABLE_COMMENT
 	COMMENT, UNCOMMENT, PREFLIGHT,
 #endif
-	CUT, CUT_TO_EOF, PASTE, INSERT, COUPLE_BEGIN, COUPLE_END, OTHER
+	ZAP, CUT, CUT_TO_EOF, PASTE, INSERT, COUPLE_BEGIN, COUPLE_END, OTHER
 } undo_type;
 
 /* Structure types. */
@@ -188,8 +188,6 @@ typedef struct colortype {
 		/* This syntax's foreground color. */
 	short bg;
 		/* This syntax's background color. */
-	bool bright;
-		/* Is this color A_BOLD? */
 	int pairnum;
 		/* The color pair number used for this foreground color and
 		 * background color. */
@@ -229,8 +227,6 @@ typedef struct syntaxtype {
 		/* The list of libmagic results that this syntax applies to. */
 	char *linter;
 		/* The command with which to lint this type of file. */
-	char *formatter;
-		/* The formatting command (for programming languages mainly). */
 #ifdef ENABLE_COMMENT
 	char *comment;
 		/* The line comment prefix (and postfix) for this type of file. */
@@ -535,7 +531,7 @@ enum
 	BOLD_TEXT,
 	QUIET,
 	SOFTWRAP,
-	POS_HISTORY,
+	POSITIONLOG,
 	LOCKING,
 	NOREAD_MODE,
 	MAKE_IT_UNIX,
@@ -544,7 +540,8 @@ enum
 	LINE_NUMBERS,
 	NO_PAUSES,
 	AT_BLANKS,
-	AFTER_ENDS
+	AFTER_ENDS,
+	LET_THEM_ZAP
 };
 
 /* Flags for the menus in which a given function should be present. */
@@ -564,11 +561,11 @@ enum
 #define MYESNO          (1<<13)
 #define MLINTER         (1<<14)
 #define MFINDINHELP     (1<<15)
-/* This is an abbreviation for all menus except Help and YesNo. */
+/* This is an abbreviation for all menus except Help and Browser and YesNo. */
 #define MMOST  (MMAIN|MWHEREIS|MREPLACE|MREPLACEWITH|MGOTOLINE|MWRITEFILE|MINSERTFILE|\
-                MEXTCMD|MBROWSER|MWHEREISFILE|MGOTODIR|MFINDINHELP|MSPELL|MLINTER)
+                MEXTCMD|MWHEREISFILE|MGOTODIR|MFINDINHELP|MSPELL|MLINTER)
 #ifndef NANO_TINY
-#define MSOME  MMOST
+#define MSOME  MMOST|MBROWSER
 #else
 #define MSOME  MMAIN|MBROWSER
 #endif
@@ -585,24 +582,32 @@ enum
 #define CONTROL_DOWN 0x404
 #define CONTROL_HOME 0x405
 #define CONTROL_END 0x406
+#define CONTROL_DELETE 0x40D
 #define SHIFT_CONTROL_LEFT 0x411
 #define SHIFT_CONTROL_RIGHT 0x412
 #define SHIFT_CONTROL_UP 0x413
 #define SHIFT_CONTROL_DOWN 0x414
 #define SHIFT_CONTROL_HOME 0x415
 #define SHIFT_CONTROL_END 0x416
+#define CONTROL_SHIFT_DELETE 0x41D
 #define ALT_LEFT 0x421
 #define ALT_RIGHT 0x422
 #define ALT_UP 0x423
 #define ALT_DOWN 0x424
+#define ALT_DELETE 0x42D
 #define SHIFT_ALT_LEFT 0x431
 #define SHIFT_ALT_RIGHT 0x432
 #define SHIFT_ALT_UP 0x433
 #define SHIFT_ALT_DOWN 0x434
+//#define SHIFT_LEFT 0x451
+//#define SHIFT_RIGHT 0x452
+#define SHIFT_UP 0x453
+#define SHIFT_DOWN 0x454
 #define SHIFT_HOME 0x455
 #define SHIFT_END 0x456
 #define SHIFT_PAGEUP 0x457
 #define SHIFT_PAGEDOWN 0x458
+#define SHIFT_DELETE 0x45D
 #define SHIFT_TAB 0x45F
 
 #ifdef USE_SLANG
@@ -622,7 +627,6 @@ enum
 #define WAS_FINAL_BACKSPACE   (1<<1)
 #define WAS_WHOLE_LINE        (1<<2)
 #define WAS_FINAL_LINE        (1<<3)
-/* The flags for the mark need to be the highest. */
 #define MARK_WAS_SET          (1<<4)
 #define WAS_MARKED_FORWARD    (1<<5)
 #endif /* !NANO_TINY */
@@ -630,9 +634,8 @@ enum
 /* The maximum number of entries displayed in the main shortcut list. */
 #define MAIN_VISIBLE (((COLS + 40) / 20) * 2)
 
-/* The default number of characters from the end of the line where
- * wrapping occurs. */
-#define CHARS_FROM_EOL 8
+/* The default number of columns from end of line where wrapping occurs. */
+#define COLUMNS_FROM_EOL 8
 
 /* The default width of a tab in spaces. */
 #define WIDTH_OF_TAB 8
