@@ -2,7 +2,7 @@
  *   search.c  --  This file is part of GNU nano.                         *
  *                                                                        *
  *   Copyright (C) 1999-2011, 2013-2018 Free Software Foundation, Inc.    *
- *   Copyright (C) 2015-2017 Benno Schulenberg                            *
+ *   Copyright (C) 2015-2018 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -101,12 +101,11 @@ void search_init(bool replacing, bool keep_the_answer)
 					answer, &search_history,
 					/* TRANSLATORS: This is the main search prompt. */
 					edit_refresh, "%s%s%s%s%s%s", _("Search"),
-					/* TRANSLATORS: The next three modify the search prompt. */
+					/* TRANSLATORS: The next four modify the search prompt. */
 					ISSET(CASE_SENSITIVE) ? _(" [Case Sensitive]") : "",
 					ISSET(USE_REGEXP) ? _(" [Regexp]") : "",
 					ISSET(BACKWARDS_SEARCH) ? _(" [Backwards]") : "", replacing ?
 #ifndef NANO_TINY
-					/* TRANSLATORS: The next two modify the search prompt. */
 					openfile->mark ? _(" (to replace) in selection") :
 #endif
 					_(" (to replace)") : "", thedefault);
@@ -156,6 +155,12 @@ void search_init(bool replacing, bool keep_the_answer)
 		} else if (func == regexp_void) {
 			TOGGLE(USE_REGEXP);
 		} else if (func == flip_replace) {
+			if (ISSET(VIEW_MODE)) {
+				print_view_warning();
+				tidy_up_after_search();
+				free(thedefault);
+				return;
+			}
 			replacing = !replacing;
 		} else {
 			if (func == flip_goto)
@@ -366,7 +371,6 @@ void do_research(void)
 	tidy_up_after_search();
 }
 
-#ifndef NANO_TINY
 /* Search in the backward direction for the next occurrence. */
 void do_findprevious(void)
 {
@@ -380,7 +384,6 @@ void do_findnext(void)
 	UNSET(BACKWARDS_SEARCH);
 	do_research();
 }
-#endif /* !NANO_TINY */
 
 /* Report on the status bar that the given string was not found. */
 void not_found_msg(const char *str)
@@ -572,19 +575,18 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 			numreplaced = 0;
 
 		if (!replaceall) {
-			size_t from_col = xplustabs();
-			size_t to_col = strnlenpt(openfile->current->data,
+			spotlighted = TRUE;
+			light_from_col = xplustabs();
+			light_to_col = strnlenpt(openfile->current->data,
 										openfile->current_x + match_len);
 
 			/* Refresh the edit window, scrolling it if necessary. */
 			edit_refresh();
 
-			spotlight(TRUE, from_col, to_col);
-
 			/* TRANSLATORS: This is a prompt. */
 			i = do_yesno_prompt(TRUE, _("Replace this instance?"));
 
-			spotlight(FALSE, from_col, to_col);
+			spotlighted = FALSE;
 
 			if (i == -1)  /* The replacing was cancelled. */
 				break;
